/*********************************************************************
	Copyright (C) 2001 by

		Alexander Berthold, alexander-berthold@web.de.
		Hoegestr. 54
		79108 Freiburg i. Breisgau
		Germany

    -- This file is part of cxTokenizer --

    "cxTokenizer" is free software; you can redistribute it and/or 
	modify it under the terms of the GNU Lesser General Public 
	License as published by the Free Software Foundation; either 
	version 2 of the License, or any later version.

    "cxTokenizer" is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
	License along with "cxTokenizer"; if not, write to the Free 
	Software  Foundation, Inc., 59 Temple Place, Suite 330, 
	Boston, MA  02111-1307  USA

    ---------------------------------------------------------------
      If you find any bugs or if you make other corrections/
	  enhancements, i'd appreciate if you'd let me know about 
	  that. My email is
  
       alexander-berthold@web.de
  
      If you share this code, do not remove this text.
    ---------------------------------------------------------------

*********************************************************************/

// cxTokenizerContext.cpp: implementation of the cxTokenizerContext class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "cxTokenizerMapData.h"
#include "cxTokenizerTokenRule.h"
#include "cxTokenizerContextCookie.h"
#include "cxTokenizerContext.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include "cxTokenizerContextDiags.cpp"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

    cxTokenizerContext::cxTokenizerContext()
	    {
	    ctkFlagsMixin<xtctx_flags>::vClearAllFlags();
	    }

    cxTokenizerContext::~cxTokenizerContext()
	    {
	    m_mapCookies.clear();
	    m_lstTokenRules.clear();
	    }

//////////////////////////////////////////////////////////////////////
// Cookie operations
//////////////////////////////////////////////////////////////////////

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::fSetCookie
	    PURPOSE:	Sets a cookie for the rule 'pltrRule'
	    RETURNS:	'true' if successful
                    'false' if cookie was already set.
    *********************************************************************/
    bool	cxTokenizerContext::fSetCookie(const cxTokenizerTokenRule *pttrRule,
									      cxTokenizerContextCookie *ptccCookie)
	    {
	    ASSERT(pttrRule!=NULL);
	    ASSERT(ptccCookie!=NULL);
	    ASSERT(pttrRule->fCheckValid());
	    ASSERT(ptccCookie->fCheckValid());

	    // TODO: find better solution
	    DWORD				dwCookieID = (DWORD)pttrRule;
	    cookiemap_iterator	it;

	    it					=m_mapCookies.find(dwCookieID);
	    if(it!=m_mapCookies.end())
		    return false;

	    ptccCookie->m_dwMyCookieID=dwCookieID;
	    m_mapCookies.insert(cookiemap_valuetype(dwCookieID,ptccCookie));
	    return true;
	    }

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::plccGetCookie
	    PURPOSE:	Returns the cookie for the ID 'dwCookieID'
	    RETURNS:	cxTokenizerContextCookie*
    *********************************************************************/
    cxTokenizerContextCookie
		    *cxTokenizerContext::ptccGetCookie(DWORD dwCookieID) const
	    {
	    ASSERT(dwCookieID!=0);

	    const_cookiemap_iterator	it;
	    it					=m_mapCookies.find(dwCookieID);
	    if(it==m_mapCookies.end())
		    return NULL;

	    ASSERT(it->second->fCheckValid());

	    return it->second;
	    }

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::plccGetCookie
	    PURPOSE:	Returns the cookie for the rule 'pltrRule'
	    RETURNS:	cxTokenizerContextCookie*
    *********************************************************************/
    cxTokenizerContextCookie
		    *cxTokenizerContext::ptccGetCookie(const cxTokenizerTokenRule *pttrRule) const
	    {
	    ASSERT(pttrRule!=NULL);
	    ASSERT(pttrRule->fCheckValid());

	    // TODO: find better solution
	    DWORD	dwCookieID = (DWORD)pttrRule;

	    return ptccGetCookie(dwCookieID);
	    }

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::fDeleteCookie
	    PURPOSE:	Deletes the cookie for the cookie ID 'dwCookieID'
	    RETURNS:	'true' if deleted
                    'false' if cookie did not exist
    *********************************************************************/
    bool	cxTokenizerContext::fDeleteCookie(DWORD dwCookieID)
	    {
	    ASSERT(dwCookieID!=0);

	    cookiemap_iterator	it;
	    it					=m_mapCookies.find(dwCookieID);
	    if(it==m_mapCookies.end())
		    return false;

	    m_mapCookies.erase(it);

	    return true;
	    }

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::fDeleteCookie
	    PURPOSE:	Deletes the cookie for the rule 'pltrRule'
	    RETURNS:	'true' if deleted
                    'false' if cookie did not exist
    *********************************************************************/
    bool	cxTokenizerContext::fDeleteCookie(cxTokenizerTokenRule *pttrRule)
	    {
	    ASSERT(pttrRule!=NULL);
	    ASSERT(pttrRule->fCheckValid());

	    // TODO: find better solution
	    DWORD	dwCookieID = (DWORD)pttrRule;

	    return fDeleteCookie(dwCookieID);
	    }

//////////////////////////////////////////////////////////////////////
// Data access operations
//////////////////////////////////////////////////////////////////////

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::nDeleteMarkedListEntries
	    PURPOSE:	Removes all rules with 'fMarkedForDeletion' - flag
                    set from the list of the currently active rules
	    RETURNS:	int; count of removed rules
    *********************************************************************/
    int		cxTokenizerContext::nDeleteMarkedListEntries()
	    {
	    // Delete the rules in the list which are marked for deletion
	    rulelist_iterator	cit,dit;
	    int					nCntDeleted;

	    nCntDeleted	=0;
	    cit			=m_lstTokenRules.begin();
	    while(cit!=m_lstTokenRules.end())
		    {
		    cxListEntry		*pteCur = *cit;
		    dit				=cit;
		    cit++;

		    if(pteCur->fIsMarkedForDeletion())
			    {
			    m_lstTokenRules.erase(dit);
			    nCntDeleted++;
			    }
		    }
	    
	    return nCntDeleted;
	    }

    /*********************************************************************
	    FUNCTION:	cxTokenizerContext::vCleanUpAfterTokenRecognition
	    PURPOSE:	Is being called after a (delimeting!) token has 
                    been recognized (cxTokenizer::vDelimTokenRecognized).
                    Resets the lexxer context.
	    RETURNS:	- void -
    *********************************************************************/
    void	cxTokenizerContext::vCleanUpAfterTokenRecognition()
	    {
	    // delete cookies
	    cookiemap_iterator	cit;
	    for(cit=m_mapCookies.begin();cit!=m_mapCookies.end();cit++)
		    delete cit->second;
	    m_mapCookies.clear();

	    // delete token list
	    rulelist_iterator	it;
	    for(it=m_lstTokenRules.begin();it!=m_lstTokenRules.end();it++)
		    (*it)->vMarkForDeletion();
	    nDeleteMarkedListEntries();

	    // delete current text
	    m_strCurrentText	=_T("");
	    }
