/*********************************************************************
	Copyright (C) 2001 by

		Alexander Berthold, alexander-berthold@web.de.
		Hoegestr. 54
		79108 Freiburg i. Breisgau
		Germany

    -- This file is part of cxTokenizer --

    "cxTokenizer" is free software; you can redistribute it and/or 
	modify it under the terms of the GNU Lesser General Public 
	License as published by the Free Software Foundation; either 
	version 2 of the License, or any later version.

    "cxTokenizer" is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
	License along with "cxTokenizer"; if not, write to the Free 
	Software  Foundation, Inc., 59 Temple Place, Suite 330, 
	Boston, MA  02111-1307  USA

    ---------------------------------------------------------------
      If you find any bugs or if you make other corrections/
	  enhancements, i'd appreciate if you'd let me know about 
	  that. My email is
  
       alexander-berthold@web.de
  
      If you share this code, do not remove this text.
    ---------------------------------------------------------------

*********************************************************************/

// cxTokenizerSTLInputStream.cpp: implementation of the cxTokenizerSTLInputStream class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "cxTokenizerInputStream.h"
#include "cxTokenizerSTLInputStream.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef _COMPILER_TREATS_EXTERNAL_TEMPLATES_CORRECTLY

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

template<class T>
cxTokenizerSTLInputStream<T>::cxTokenizerSTLInputStream(T& stlIStream)
:	m_stlIStream(stlIStream)
	{
	m_strPutBackData=_T("");
	m_nPutBackPos	=0;

	m_tcNextChar	='\0';
	m_fEof			=false;

	vGetNextChar();
	}

template<class T>
cxTokenizerSTLInputStream<T>::~cxTokenizerSTLInputStream()
	{
	}

//////////////////////////////////////////////////////////////////////
// cxTokenizerInputStream Protected operations
//////////////////////////////////////////////////////////////////////

template<class T>
void	cxTokenizerSTLInputStream<T>::vGetNextChar()
	{
	ASSERT(m_fEof==false);
	T::int_type		tmp;

	tmp				=m_stlIStream.get();
	if(tmp==T::eof())
		m_fEof			=true;
	else
		{
		m_tcNextChar	=(TCHAR)tmp;
		}
	
	}

//////////////////////////////////////////////////////////////////////
// cxTokenizerInputStream Operations
//////////////////////////////////////////////////////////////////////

template<class T>
bool	cxTokenizerSTLInputStream<T>::fCheckValid() const
	{
	return true;
	}

template<class T>
TCHAR	cxTokenizerSTLInputStream<T>::tcGetNextCharacter()
	{
	bool	fEof = fIsEofReached();
	int		nPutBackLength = m_strPutBackData.length();
	TCHAR	tcReturn = _T('\0');

	ASSERT(!fEof);
	if(fEof)
		throw cxTokenizerException(ERR_READ_PAST_EOF);

	if(m_nPutBackPos < nPutBackLength)
		{
		tcReturn	=m_strPutBackData[m_nPutBackPos++];
		if(m_nPutBackPos == nPutBackLength)
			{
			m_nPutBackPos	=0;
			m_strPutBackData=_T("");
			}
		}
	else
		{
		tcReturn	=m_tcNextChar;
		vGetNextChar();
		}

	return tcReturn;
	}

template<class T>
bool	cxTokenizerSTLInputStream<T>::fPutBack(LPCTSTR lpszText)
	{
	m_strPutBackData.insert(0,lpszText);
	return true;
	}

template<class T>
bool	cxTokenizerSTLInputStream<T>::fPutBack(TCHAR tcChar)
	{
	m_strPutBackData.insert(0,tcChar);
	return true;
	}

template<class T>
bool	cxTokenizerSTLInputStream<T>::fIsEofReached() const
	{
	int		nPutBackLength = m_strPutBackData.length();

	if(m_nPutBackPos<nPutBackLength))
		return false;

	return m_fEof;
	}

template<class T>
bool	cxTokenizerSTLInputStream<T>::fShouldDelete() const
	{
	return false;
	}

#endif