/*********************************************************************
	Copyright 2001 Alexander Berthold, alexander-berthold@web.de.

    -- This file is part of ctkCommon --

    "ctkCommon" is free software; you can redistribute it and/or 
	modify it under the terms of the GNU Lesser General Public 
	License as published by the Free Software Foundation; either 
	version 2 of the License, or any later version.

    "ctkCommon" is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
	License along with "ctkCommon"; if not, write to the Free 
	Software  Foundation, Inc., 59 Temple Place, Suite 330, 
	Boston, MA  02111-1307  USA

    ---------------------------------------------------------------
      If you find any bugs or if you make other corrections/
	  enhancements, i'd appreciate if you'd let me know about 
	  that. My email is
  
       alexander-berthold@web.de
  
      If you share this code, do not remove this text.
    ---------------------------------------------------------------

Class:      ctkFlagsMixin
Author:     Alexander Berthold
Copyright:  Alexander Berthold
Date:       2001/06/05
Version:	0.1.01
Purpose:    Mixin class for flag functionality.


Version history:

	-	2001/06/05
		Current source labeled version 0.1.01
		
*********************************************************************/

// ctkFlagsMixin.h: interface for the ctkFlagsMixin class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CTKFLAGSMIXIN_H__29B1A8C1_F9B4_4725_BE18_31BFFBD0F658__INCLUDED_)
#define AFX_CTKFLAGSMIXIN_H__29B1A8C1_F9B4_4725_BE18_31BFFBD0F658__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

template<class T>
class ctkFlagsMixin
{
// Construction/Destruction
public:
	ctkFlagsMixin()
		{
		m_flags=(T)0;
		}
	ctkFlagsMixin(const ctkFlagsMixin<T>& other)
		{
		m_flags	=other.m_flags;
		}
	ctkFlagsMixin(T mask)
		{
		m_flags	=mask;
		}
	//virtual ~ctkFlagsMixin();

// Attributes
public:
	unsigned long	m_flags;

// Operations
public:
	bool	fIsFlagSet(T mask) const
		{
		if( ((T)(m_flags&mask))==mask )
			return true;
		else
			return false;
		}

	void	vSetFlag(T mask, bool fSet = true)
		{
		if(!fSet)
			m_flags	&=~mask;
		else
			m_flags	|=mask;
		}

	void	vResetFlag(T mask)
		{
		return vSetFlag(mask,false);
		}

	void	vClearAllFlags()
		{
		m_flags	=(T)0;
		}

	bool	operator==(T mask) const
		{
		return m_flags==mask;
		}

	bool	operator==(ctkFlagsMixin<T>& other) const
		{
		return m_flags==other.m_flags;
		}

	bool	operator!=(T mask) const
		{
		return m_flags!=mask;
		}

	bool	operator!=(ctkFlagsMixin<T>& other) const
		{
		return m_flags!=other.m_flags;
		}

	ctkFlagsMixin&	operator|=(T mask)
		{
		vSetFlag(mask);
		return *this;
		}
};

#endif // !defined(AFX_CTKFLAGSMIXIN_H__29B1A8C1_F9B4_4725_BE18_31BFFBD0F658__INCLUDED_)
